@@ -15,6 +15,7 @@ from fa_purity.frozen import (
 from fa_purity.pure_iter._inner import (
     InnerPureIter,
 )
+import functools
 import more_itertools
 from typing import (
     Callable,
@@ -34,6 +35,7 @@ def _chunked(items: Iterable[_T], size: int) -> Iterator[FrozenList[_T]]:
 
 @dataclass(frozen=True)
 class PureIter(Generic[_T]):
+    # unsafe_unwrap use is safe due to iters equivalence
     _inner: InnerPureIter[_T]
 
     def map(self, function: Callable[[_T], _R]) -> PureIter[_R]:
@@ -42,6 +44,19 @@ class PureIter(Generic[_T]):
         )
         return PureIter(draft)
 
+    def reduce(self, function: Callable[[_R, _T], _R], init: _R) -> _R:
+        return unsafe_unwrap(
+            self._inner.new_iter.map(
+                lambda i: functools.reduce(function, i, init)
+            )
+        )
+
+    def filter(self, function: Callable[[_T], bool]) -> PureIter[_T]:
+        draft = InnerPureIter(
+            self._inner.new_iter.map(lambda i: iter(filter(function, i)))
+        )
+        return PureIter(draft)
+
     def chunked(self, size: int) -> PureIter[FrozenList[_T]]:
         draft = InnerPureIter(
             self._inner.new_iter.map(lambda i: _chunked(i, size))
@@ -55,5 +70,4 @@ class PureIter(Generic[_T]):
         return function(self)
 
     def __iter__(self) -> Iterator[_T]:
-        # all cmds will result in an equivalent new iterator
         return iter(unsafe_unwrap(self._inner.new_iter))
