@@ -14,6 +14,7 @@ from fa_purity.cmd import (
 from fa_purity.frozen import (
     FrozenList,
 )
+import functools
 from typing import (
     Callable,
     Generic,
@@ -42,6 +43,17 @@ class Stream(_Stream[_T]):
         )
         return Stream(draft)
 
+    def reduce(self, function: Callable[[_R, _T], _R], init: _R) -> Cmd[_R]:
+        return self._new_iter.map(
+            lambda i: functools.reduce(function, i, init)
+        )
+
+    def filter(self, function: Callable[[_T], bool]) -> Stream[_T]:
+        draft = _Stream(
+            self._new_iter.map(lambda i: iter(filter(function, i)))
+        )
+        return Stream(draft)
+
     def chunked(self, size: int) -> Stream[FrozenList[_T]]:
         draft = _Stream(
             self._new_iter.map(lambda i: _iter_factory.chunked(i, size))
